@@ -6,7 +6,8 @@ Library of Medicine (NLM), National Institutes of Health, Department of Health
  endorse or recommend this or any other product."
 """
 from .base import Base
-from therapy import PROJECT_ROOT, DownloadException, OTHER_IDENTIFIERS, XREFS
+from therapy import PROJECT_ROOT, DownloadException, XREF_SOURCES, \
+    ASSOC_WITH_SOURCES, ITEM_TYPES
 import therapy
 from therapy.database import Database
 from therapy.schemas import SourceName, NamespacePrefix, SourceMeta, Drug, \
@@ -14,8 +15,6 @@ from therapy.schemas import SourceName, NamespacePrefix, SourceMeta, Drug, \
 import csv
 import datetime
 import logging
-import botocore
-from typing import List
 from os import environ, remove
 import subprocess
 import shutil
@@ -48,27 +47,17 @@ class RxNorm(Base):
 
     def __init__(self,
                  database: Database,
+                 data_path=PROJECT_ROOT / 'data',
                  data_url='https://www.nlm.nih.gov/research/umls/'
                           'rxnorm/docs/rxnormfiles.html'):
         """Initialize the RxNorm ETL class.
 
         :param Database database: Access to DynamoDB.
+        :param Path data_path: path to app data directory
         :param str data_url: URL to RxNorm data files.
         """
+        super().__init__(database, data_path)
         self._data_url = data_url
-        self.database = database
-        self._added_ids = []
-
-    def perform_etl(self) -> List[str]:
-        """Load the RxNorm source into database.
-
-        :return: List of concept IDs which were successfully processed and
-            uploaded.
-        """
-        self._extract_data()
-        self._load_meta()
-        self._transform_data()
-        return self._added_ids
 
     def _extract_data(self, *args, **kwargs):
         """Extract data from the RxNorm source."""
@@ -191,7 +180,7 @@ class RxNorm(Base):
                             self._add_str_field(params, row,
                                                 precise_ingredient,
                                                 drug_forms, sbdfs)
-                            self._add_other_ids_xrefs(params, row)
+                            self._add_xref_assoc(params, row)
                             data[concept_id] = params
                         else:
                             # Concept already created
@@ -199,7 +188,7 @@ class RxNorm(Base):
                             self._add_str_field(params, row,
                                                 precise_ingredient,
                                                 drug_forms, sbdfs)
-                            self._add_other_ids_xrefs(params, row)
+                            self._add_xref_assoc(params, row)
 
             with self.database.therapies.batch_writer() as batch:
                 for key, value in data.items():
@@ -208,118 +197,16 @@ class RxNorm(Base):
                                               ingredient_brands, sbdfs)
                         self._load_brand_concepts(value, brands, batch)
 
-                        params = Drug(
-                            concept_id=value['concept_id'],
-                            label=value['label'] if 'label' in value else None,
-                            approval_status=value[
-                                'approval_status'] if 'approval_status'
-                                                      in value else None,
-                            aliases=value['aliases'] if 'aliases' in
-                                                        value else [],
-                            other_identifiers=value[
-                                'other_identifiers'] if 'other_identifiers'
-                                                        in value else [],
-                            xrefs=value['xrefs'] if 'xrefs' in value else [],
-                            trade_names=value['trade_names'] if 'trade_names'
-                                                                in value
-                                                                else []
-                        )
-                        self._load_therapy(params, batch)
+                        params = {
+                            'concept_id': value['concept_id']
+                        }
 
-    def _load_therapy(self, params, batch):
-        """Load therapy record into the database.
-
-        :param Drug params: A Therapy object.
-        :param BatchWriter batch: Object to write data to DynamoDB.
-        """
-        params = dict(params)
-        for label_type in ['label', 'aliases', 'other_identifiers', 'xrefs',
-                           'trade_names']:
-            if not params[label_type]:
-                del params[label_type]
-
-        for attr in ['trade_names', 'aliases']:
-            # Remove duplicates
-            if attr in params:
-                if 'label' in params:
-                    params[attr] = list(set(params[attr]) - {params['label']})
-
-                if attr == 'aliases' and 'trade_names' in params:
-                    params[attr] = \
-                        list(set(params[attr]) - set(params['trade_names']))
-
-                # 20 cutoff for aliases / trade names
-                if len(params[attr]) > 20:
-                    logger.debug(f"{params['concept_id']} has more than 20"
-                                 f" {attr}.")
-                    del params[attr]
-
-        self._load_label_types(params, batch)
-        params['src_name'] = SourceName.RXNORM.value
-        params['label_and_type'] = f"{params['concept_id'].lower()}##identity"
-        params['item_type'] = 'identity'
-        try:
-            batch.put_item(Item=params)
-            self._added_ids.append(params['concept_id'])
-        except botocore.exceptions.ClientError:
-            if (len((params['label_and_type']).encode('utf-8')) >= 2048) or \
-                    (len((params['concept_id']).encode('utf-8')) >= 1024):
-                logger.info(f"{params['concept_id']}: An error occurred "
-                            "(ValidationException) when calling the "
-                            "BatchWriteItem operation: Hash primary key "
-                            "values must be under 2048 bytes, and range"
-                            " primary key values must be under 1024"
-                            " bytes.")
-
-    def _load_label_types(self, params, batch):
-        """Insert aliases, trade_names, and label data into the database.
-
-        :param dict params: A transformed therapy record.
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        if 'label' in params:
-            self._load_label_type(params, batch, 'label', 'label')
-        if 'trade_names' in params:
-            self._load_label_type(params, batch, 'trade_name', 'trade_names')
-        if 'aliases' in params:
-            self._load_label_type(params, batch, 'alias', 'aliases')
-        if 'other_identifiers' in params:
-            self._load_label_type(params, batch, 'other_id',
-                                  'other_identifiers')
-        if 'xrefs' in params:
-            self._load_label_type(params, batch, 'xref', 'xrefs')
-
-    def _load_label_type(self, params, batch, label_type_sing, label_type_pl):
-        """Insert alias, trade_name, or label data into the database.
-
-        :param dict params: A transformed therapy record.
-        :param BatchWriter batch: Object to write data to DynamoDB
-        :param str label_type_sing: The singular label type
-        :param str label_type_pl: The plural label type
-        """
-        if isinstance(params[label_type_pl], list):
-            terms = {t.casefold(): t for t in params[label_type_pl]}.values()
-        else:
-            terms = [params[label_type_pl]]
-        for t in terms:
-            t = {
-                'label_and_type': f"{t.lower()}##{label_type_sing}",
-                'concept_id': f"{params['concept_id'].lower()}",
-                'src_name': SourceName.RXNORM.value,
-                'item_type': label_type_sing
-            }
-            try:
-                batch.put_item(Item=t)
-            except botocore.exceptions.ClientError:
-                if (len((t['label_and_type']).encode('utf-8')) >= 2048) or \
-                        (len((t['concept_id']).encode('utf-8')) >= 1024):
-                    logger.info(f"{params['concept_id']}: An error occurred "
-                                "(ValidationException) when calling the "
-                                "BatchWriteItem operation: Hash primary key "
-                                "values must be under 2048 bytes, and range"
-                                " primary key values must be under 1024"
-                                " bytes.")
-        return terms
+                        for field in list(ITEM_TYPES.keys()) + ['approval_status']:  # noqa: E501
+                            field_value = value.get(field)
+                            if field_value:
+                                params[field] = field_value
+                        assert Drug(**params)
+                        self._load_therapy(params)
 
     def _get_brands(self, row, ingredient_brands):
         """Add ingredient and brand to ingredient_brands.
@@ -373,7 +260,8 @@ class RxNorm(Base):
             for tn in sbdfs[record_label]:
                 self._add_term(value, tn, 'trade_names')
 
-    def _load_brand_concepts(self, value, brands, batch):
+    @staticmethod
+    def _load_brand_concepts(value, brands, batch):
         """Connect brand names to a concept and load into the database.
 
         :params dict value: A transformed therapy record
@@ -399,7 +287,7 @@ class RxNorm(Base):
         :param list row: A row in the RxNorm data file.
         :param dict precise_ingredient: Precise ingredient information
         :param list drug_forms: RxNorm Drug Form values
-        :param dict precise_ingredient: Brand names for precise ingredient
+        :param dict sbdfs: Brand names for precise ingredient
         """
         term = row[14]
         term_type = row[12]
@@ -432,7 +320,8 @@ class RxNorm(Base):
             elif term_type == 'PEP':
                 self._add_term(precise_ingredient, term, row[13])
 
-    def _add_term(self, params, term, label_type):
+    @staticmethod
+    def _add_term(params, term, label_type):
         """Add a single term to a therapy record in an associated field.
 
         :param dict params: A transformed therapy record.
@@ -445,25 +334,25 @@ class RxNorm(Base):
         else:
             params[label_type] = [term]
 
-    def _add_other_ids_xrefs(self, params, row):
-        """Add other identifier or xref to therapy.
+    def _add_xref_assoc(self, params, row):
+        """Add xref or associated_with to therapy.
 
         :param dict params: A transformed therapy record.
         :param list row: A row in the RxNorm data file.
         """
         if row[11]:
-            other_id_xref = row[11].upper()
-            if other_id_xref in OTHER_IDENTIFIERS:
+            xref_assoc = row[11].upper()
+            if xref_assoc in XREF_SOURCES:
                 source_id =\
-                    f"{NamespacePrefix[other_id_xref].value}:{row[13]}"
+                    f"{NamespacePrefix[xref_assoc].value}:{row[13]}"
                 if source_id != params['concept_id']:
                     # Sometimes concept_id is included in the source field
-                    self._add_term(params, source_id, 'other_identifiers')
-            elif other_id_xref in XREFS:
-                source_id = f"{NamespacePrefix[other_id_xref].value}:{row[13]}"
-                self._add_term(params, source_id, 'xrefs')
+                    self._add_term(params, source_id, 'xrefs')
+            elif xref_assoc in ASSOC_WITH_SOURCES:
+                source_id = f"{NamespacePrefix[xref_assoc].value}:{row[13]}"
+                self._add_term(params, source_id, 'associated_with')
             else:
-                logger.info(f"{other_id_xref} not in NameSpacePrefix.")
+                logger.info(f"{xref_assoc} not in NameSpacePrefix.")
 
     def _load_meta(self):
         """Add RxNorm metadata."""
